'--------------------------------------------------------------------------------------
'name                     : CalibrateInternalRCOscillatorATmega169.bas
'programmed by            : Claus Kuhnel
'date                     : 2008-07-14
'purpose                  : Calibrate Internal RC Oscillator on AVR Butterfly
'micro                    : ATmega169
'tested with              : V.1.12.0.0 (2010-01-20)
'suited for demo          : yes
'commercial addon needed  : no
'--------------------------------------------------------------------------------------
$regfile = "m169def.dat"                                    ' specify the used micro
$crystal = 8000000                                          ' used crystal frequency
$baud = 19200                                               ' used baud rate

$hwstack = 40                                               ' default use 32 for the hardware stack
$swstack = 40                                               ' default use 10 for the SW stack
$framesize = 40                                             ' default use 40 for the frame space

$lib "lcd_butterfly.lbx"

'Butterfly LCD Setup:
Dim _butterfly_digit As Byte

Const I1 = &H40
Const I2 = &H70                                             ' &H45 was read out value
Const Count = 6103
Const Timer2counts = 200

Dim I As Byte
Dim Osccal_opt As Byte
Dim Tmp1 As Word
Dim Tmp2 As Integer
Dim Minimum As Integer : Minimum = 32767

Declare Sub Timerinit()                                     ' initialize Timer1 & Timer2
Declare Function Calibrate(osccal_value As Byte) As Word


Main:
   Timerinit
   Cls : Lcd "CAL-RC"
   Print "Calibration of internal RC oscillator"
   Waitms 500
   For I = I1 To I2
      nop
      Tmp1 = Calibrate(i)
      Tmp2 = Tmp1 - Count
      Tmp2 = Abs(tmp2)
      If Tmp2 < Minimum Then
         Minimum = Tmp2
         Osccal_opt = I
      End If
      Print Hex(i) ; Spc(3) ; Tmp1 ; Spc(3) ; Tmp2 ; Spc(3) ; Minimum
   Next
   Osccal = Osccal_opt
   Waitms 100
   Print
   Print "Optimal OSCCAL = 0x" ; hex(Osccal_opt)
End

Sub Timerinit()
   Ocr2a = Timer2counts                                     ' Count value for Timer2
   Set Assr.as2                                             ' 32 kHz Clock for Timer2
   Set Tccr2a.wgm21                                         ' Timer2 CTC Mode, OC2A disconnected.
   Set Tccr1b.wgm12
End Sub

Function Calibrate(osccal_value As Byte) As Word
   Osccal = Osccal_value                                    ' set Oscillator Calibration Register
   Tcnt1 = 0 : Tcnt2 = 0
   Set Tccr2a.cs20                                          ' Timer2 no Prescaler     -> Start
   Set Tccr1b.cs11                                          ' Counter1 Prescaler = 8  -> Start
   Do
   Loop Until Tifr2.ocf2a = 1
   Calibrate = Tcnt1
   Reset Tccr2a.cs20                                        ' Timer2                  -> Stop
   Reset Tccr1b.cs11                                        ' Counter1                -> Stop
   Set Tifr2.ocf2a
End Function

$include "Butterfly_lcd_data.bas"

