'--------------------------------------------------------------------------------------
'name                     : Stepper.bas
'programmed by            : Claus Kuhnel
'date                     : 2010-10-29
'purpose                  : Stepper Motor Control w/ EasyDriver 4.2
'micro                    : ATmega48
'tested with              : V.1.12.0.0
'suited for demo          : yes
'commercial addon needed  : no
'--------------------------------------------------------------------------------------

$regfile = "m168def.dat"
$crystal = 20000000                                         ' used crystal frequency
$baud = 9600                                                ' used baud rate

$hwstack = 40                                               ' default use 32 for the hardware stack
$swstack = 40                                               ' default use 10 for the SW stack
$framesize = 60                                             ' default use 40 for the frame space

' myAVR LCD Configuration
Config Lcdpin = Pin , Db4 = Portd.4 , Db5 = Portd.5 , Db6 = Portd.6 , Db7 = Portd.7 , E = Portd.3 , Rs = Portd.2
Config Lcd = 16 * 2
Cursor Off

Config Portb = Output

Const Fullstep = 0
Const Halfstep = 1
Const Quarterstep = 2
Const Eightstep = 3

Const Forward = 0
Const Backward = 1

Const Fast = 0
Const Slow = 1

P_step Alias Portb.3 : Reset P_step
P_direction Alias Portb.2
P_m2 Alias Portb.1
P_m1 Alias Portb.0

Declare Sub Stepper1(byval Steps As Word , Byval Stepdelay As Byte , Byval Direction As Byte , Byval Mde As Byte)
Declare Sub Stepper2(byval Steps As Word , Byval Speed As Byte , Byval Direction As Byte, Byval Mde As Byte)

Cls
Lcd "Stepper Control"

Do
   Lowerline
   Lcd "800 1/2 forward."
   Call Stepper1(800 , 2 , Forward , Halfstep)
   Wait 1
   Lowerline
   Lcd "3200 1/8 backw. "
   Stepper1 3200 , 0 , Backward , Eightstep
   Wait 1
   Lowerline
   Lcd "400 1/1 fastfwd."
   Call Stepper2(400 , Fast , Forward , Fullstep)
   Wait 1
   Lowerline
   Lcd "800 1/2 slowback"
   Stepper2 800 , Slow , Backward , Halfstep
   Wait 1
Loop

End

Sub Stepper1(byval Steps As Word , Byval Stepdelay As Byte , Byval Direction As Byte , Byval Mde As Byte)
   P_m2 = Mde.1
   P_m1 = Mde.0
   P_direction = Direction

   Local I As Word
   Shift Stepdelay , Right , 1
   If Stepdelay = 0 Then Stepdelay = 1

   For I = 0 To Steps
      Set P_step
      Waitus 1 : Waitms Stepdelay
      Reset P_step
      Waitus 1 : Waitms Stepdelay
   Next

End Sub

Sub Stepper2(byval Steps As Word , Byval Speed As Byte , Byval Direction As Byte , Byval Mde As Byte)
   P_m2 = Mde.1
   P_m1 = Mde.0
   P_direction = Direction

   Local I As Word

   For I = 0 To Steps
      Set P_step
      Waitms 1
      If Speed <> 0 Then Waitms 2
      Reset P_step
      Waitms 1
      If Speed <> 0 Then Waitms 2
   Next

End Sub