'-------------------------------------------------------------------------------------
'name                     : TempButterfly.bas
'programmed by            : Claus Kuhnel
'date                     : 2009-01-22
'purpose                  : Test of Temperature Measurment on AVR Butterfly
'micro                    : ATmega169
'tested with              : V.1.12.0.0 (2010-10-20)
'suited for demo          : yes
'commercial addon needed  : no
'-------------------------------------------------------------------------------------
$regfile = "m169def.dat"                                    ' specify the used micro
$crystal = 8000000                                          ' used crystal frequency
$baud = 19200                                               ' used baud rate

$hwstack = 40                                               ' default use 32 for the hardware stack
$swstack = 20                                               ' default use 10 for the SW stack
$framesize = 40                                             ' default use 40 for the frame space

$lib "lcd_butterfly.lbx"

Debug On                                                    ' set debug on for UART output

Const Temperature = 0                                       ' MUX address for temperatur sensor

'Butterfly LCD Setup:
Dim _butterfly_digit As Byte

Dim S As String * 12
Dim Result As Word
Dim Tabvalue As Word
Dim Temp As Byte
Dim I As Byte
Dim Wtemp As Word

$include "CalibrateInternalRCOscillatorATmega169.inc"
'Osccal = &H49                                            ' for test purposes only

Cls : Lcd "TEMP"
Debug "" : Debug "Optimal OSCCAL = 0x" ;
Debug Hex(osccal_opt)
Debug "Measuring Temperature on AVR Butterfly..."

Config Adc = Single , Prescaler = Auto , Reference = Avcc
'Reference = 3.3                   ' Reference voltage is AVCC
Start Adc
Do
   Result = Getadc(temperature)
   Cls : Lcd Str(result)
   Debug "Temperature = " ; Result ; Spc(3);
   Waitms 500
   S = ""
   Select Case Result
      Case Is > 810 : Gosub Neg_temp
      Case Is < 800 : Gosub Pos_temp
      Case Else : Gosub Zero_temp
   End Select
   Debug Temp ; " C"
   Waitms 500
Loop
End

Neg_temp:                                                   ' Temperature is negative
   Restore Temp_celcius_neg
   For I = 0 To 14
      Read Tabvalue
      If Result <= Tabvalue Then
            Temp = I : Exit For
      End If
   Next
   S = "-"
   S = S + Str(temp)
   S = S + "*C"
   Cls : Lcd S
Return

Pos_temp:                                                   ' Temperature is positive
   Restore Temp_celcius_pos
   For I = 0 To 60
      Read Tabvalue
      If Result >= Tabvalue Then
         Temp = I : Exit For
      End If
   Next
   S = "+"
   S = S + Str(temp)
   S = S + "*C"
   Cls : Lcd S
Return

Zero_temp:
   Temp = 0
   S = Str(temp)
   S = S + "*C"
   Cls : Lcd S
Return

$include "Butterfly_lcd_data.bas"

Temp_celcius_pos:                                           ' Positive Celcius temperatures (ADC-value) from 0 to 60 degrees
Data 806% , 796% , 786% , 775% , 765% , 754% , 743% , 732% , 720% , 709% , 697% , 685% , 673% , 661% , 649%
Data 636% , 624% , 611% , 599% , 586% , 574% , 562% , 549% , 537% , 524% , 512% , 500% , 488% , 476% , 464%
Data 452% , 440% , 429% , 418% , 406% , 396% , 385% , 374% , 364% , 354% , 344% , 334% , 324% , 315% , 306%
Data 297% , 288% , 279% , 271% , 263% , 255% , 247% , 240% , 233% , 225% , 219% , 212% , 205% , 199% , 193%
Data 187%

Temp_celcius_neg:                                           ' Negative Celcius temperatures (ADC-value) from -1 to -15 degrees
Data 815% , 825% , 834% , 843% , 851% , 860% , 868% , 876% , 883% , 891% , 898% , 904% , 911% , 917% , 923%