'------------------------------------------------------------------------------------
'name                     : adc3sS.bas
'programmed by            : Claus Kuhnel
'date                     : 2008-12-28
'purpose                  : Single-ended  AD conversion using int. reference voltage
'                           Timer1 triggered
'micro                    : ATmega32
'tested with              : V.1.12.0.0 (2010-08-08)
'suited for demo          : yes
'commercial addon needed  : no
'------------------------------------------------------------------------------------
$regfile = "m32def.dat"
$crystal = 14745600
$baud = 19200

$include "init_staver24.bas"

Yellowled Alias Portc.3
Set Yellowled  ' switch yellow led off

Const Bandgap = &B11110  ' MUX address for bandgap reference
Const 1_sec_reload = &HC7C0 ' Timer1 reload value

Dim Result As Word
Dim Temp As Word
Dim Voltage As Single
Dim Reference As Single
Dim S As String * 10
Dim Adc_ready As Bit


Declare Function Readadc() As Word   ' read ADC result from ADCH & ADCL

On Adcc Adcinterrupt     ' interrupt vector for adc ready interrupt
On Ovf1 Timerinterrupt     ' interrupt vector for timer1 overflow interrupt


' Configuration of ADC for autotrigger by timer1 overflow
Admux = &H5E      ' AVCC is reference, bandgap is measured
Adcsr = &HA7      ' prescaler 128, ADC enabled, auto trigger
Sfior = &B11000000      ' select timer1 overflow as autotrigger

Reference = 5.0       ' Reference voltage is AVCC


Config Timer1 = Timer , Prescale = 1024  ' Timer0 Configuration
Set Sfior.psr10                          ' reset prescaler
Timer1 = 1_sec_reload

Enable Ovf1        ' enable timer0 overflow interrupt
Set Adcsr.adie       ' enable adc interrupt
Enable Interrupts       ' enable global interrupt

Adcsr = Adcsr Or &H40      ' start first AD conversion

Do
  If Adc_ready = 1 Then      ' is there a new adc result?
    Result = Readadc()
    Print
    Print "ADC  = " ; Hex(result) ; Spc(3);

    Voltage = Result * Reference
    Voltage = Voltage / 1024

    S = Fusing(voltage , " #.###")
    Print "Vref = " ; S ; " V"

    Adc_ready = 0
  Else
    Print ".";
    Waitms 100
  End If

Loop

End

Function Readadc()       ' read ADC result from ADCH & ADCL
  Local W1 As Word
  Local W2 As Word

  W1 = Adcl                            ' read Lo byte frist
  W2 = Adch                            ' read Hi byte afterwards
  Shift W2 , Left , 8
  Readadc = W1 + W2
End Function

Timerinterrupt:
  Timer1 = 1_sec_reload
Return

Adcinterrupt:
  Adc_ready = 1       ' set adc ready flag for main loop
  Reset Yellowled       ' blink yellow led
  Waitms 10
  Set Yellowled
Return