'-------------------------------------------------------------------
' iButton Zutrittskontrolle
'-------------------------------------------------------------------
$regfile = "m8def.dat"                                      ' ATmega8
$crystal = 3686400                                          ' fr STK500
$baud = 9600

Config 1wire = Portb.0                                      ' PB0 ist 1-Wire Busleitung

Declare Sub Crc8_check()                                    ' Berechne CRC-8 fr ROM Inhalt
Declare Function Searchid() As Byte                         ' Suche ID in der Datenbank

Const _debug = 1
Const Ids = 5                                               ' Anzahl von IDs in der Datenbank

Dim I As Byte                                               ' Index Variable
Dim Rom_data(8) As Byte                                     ' ROM Daten vom iButton
Dim Compare(8) As Byte                                      ' Vergleichsergebnisse
Dim Id As Byte                                              ' Laufende ID (1 bis IDs)
Dim Value As Byte

Do
   Print "Bitte Batch kontaktieren."
   Do
      Rom_data(1) = 1wsearchfirst()                         ' Suche nach iButton
   Loop Until Err = 0
   Print "Batch detektiert."
#if _debug
   Print "Dallas 1-Wire ID : "
   For I = 1 To 8
      Print Hex(rom_data(i));                               ' Ausgabe der ROM Daten
   Next
   Crc8_check                                               ' berprfung CRC-8
   Print
#endif
   Print "Suche in Datenbank... "
   If Searchid() = 0 Then
      Print "ID gefunden."
   Else
      Print "ID nicht gefunden."
   End If
   Do
      Rom_data(1) = 1wsearchfirst()                         ' Warten bis iButton entfernt ist
   Loop Until Err = 1
Loop

End

Function Searchid()
   Local Result As Byte
   Restore Id_tab
   Id = 1                                                   ' beginne mit erster ID
   While Id <= Ids
      Print "*";
      For I = 1 To 8
         Read Value                                         ' Lesen IDByte aus Datenbank
         If Value = Rom_data(i) Then                        ' Byteweiser Vergleich
            Compare(i) = 0
         Else
            Compare(i) = 1
         End If
      Next
      Result = 0
      For I = 1 To 8
         Result = Result + Compare(i)
      Next
      If Result = 0 Then Exit While                         ' wenn ID gefunden dann Ende
      Incr Id
   Wend
   Print
   Searchid = Result
End Function

Sub Crc8_check()
   $external _crc8
   Local Result As Byte
   Loadadr Rom_data(1) , Z
   ldi R24, 8
   !Call _crc8
   Loadadr Result , X
   st X, R16
   If Result = 0 Then
      Print " CRC-8 OK";
   Else
      Print " CRC-8 ERROR";
   End If
End Sub

' -----[ Data ]-----------------------------------------------
'
Id_tab:
Data &H04 , &H4B , &H7F , &H49 , &H00 , &H00 , &H00 , &H08
Data &H10 , &HA7 , &HB2 , &H54 , &H00 , &H00 , &H00 , &H38
Data &H10 , &H37 , &H84 , &H11 , &H00 , &H00 , &H00 , &HF2
Data &H10 , &H37 , &H84 , &H11 , &H00 , &H00 , &H00 , &HF0
Data &H10 , &H37 , &H84 , &H11 , &H00 , &H00 , &H00 , &HF0