'------------------------------------------------------------------------------------
' Measuring Temperature and Humidity by stAVeR24
'
' Sensor for Temperature DS2438 (MAXIM/Dallas)
' Sensor for Humidity HIH-3610 (Honeywell)
'
' 2005-12-12 Claus Kuhnel
'------------------------------------------------------------------------------------
$regfile = "M32DEF.DAT"                                     ' stAVeR-24M32 has the ATMega32 on board
$crystal = 14745600                                         ' 14.7456 MHz oscillator
$baud = 38400                                               ' USART (RS232) Speed
'$sim ' uncomment for simulation

Yellowled Alias Portc.3 ' YELLOW LED

Config 1wire = Portd.4

Const _dbg = 0                                              ' Switch Compiler Condition (0 = no debug info)
Const Ds2438 = &H26
Const Skip_rom = &HCC                                       ' Skip ROM Commands
Const Read_rom = &H33                                       ' Read ROM
Const Match_rom = &H55                                      ' Match ROM
Const Search_rom = &HF0                                     ' Search ROM

' this commands must follow a page byte [0...7]
Const Convertt = &H44                                       ' Start Temperature Conversion
Const Convertv = &HB4                                       ' Start Voltage Conversion
Const Read_scratch = &HBE                                   ' Read Scratchpad RAM
Const Write_scratch = &H4E                                  ' Write Scratchpad RAM
Const Copy_scratch = &H48                                   ' Copy Scratchpad RAM
Const Recall_memory = &HB8                                  ' Copy Memory to Scratchpad RAM
Const Measurevdd = &B00001000                               ' Setup for Status Register
Const Measurevad = &B00000000

Dim Scratch(9) As Byte                                      ' Scratchpad RAM Copy
Dim I As Byte , W As Word
Dim Done As Byte
Dim Itemp As Integer
Dim Temp As Single
Dim Vdd As Single
Dim Vad As Single
Dim Rh_raw As Single
Dim Rh_true As Single
Dim Count As Word

Declare Sub _convertt()
Declare Sub _convertv()
Declare Sub _recallmemory(byval Page As Byte)
Declare Sub _readscratch(byval Page As Byte)
Declare Sub _writescratch(byval Page As Byte)
Declare Sub _copyscratch(byval Page As Byte)
Declare Sub _clearscratch(byval Page As Byte)
Declare Function Initialize() As Byte
Declare Sub Measure()
Declare Sub Crc8_check()

Gosub Init_stAVeR24

Cls : Waitms 500

Reset Yellowled

Upperline
Lcd "DS2438 Test"
Set Yellowled
#if _dbg
Print : Print "DS2438 Test"
#endif
Wait 1
Done = Initialize()
If Done <> 1 Then
   Cls
   Lcd "Program aborted"
#if _dbg
   Select Case Done
      Case 0 : Print "No 1wire device found"
      Case Else : Print "Only one 1wire device supported by this program version"
   End Select
#endif
   End
End If
Wait 3 : Cls
Do
   Reset Yellowled
   Waitms 50
   Measure
   Set Yellowled
   Wait 5
Loop

End

$include "init_stAVeR24.bas"

'------------------------------------------------------------------------------------
' Subroutines & Functions
'------------------------------------------------------------------------------------
Function Initialize() As Byte
   Count = 0
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Read_rom                                         ' Read ROM Command
#if _dbg
   For I = 1 To 8
      Scratch(i) = 1wread()
      Print Hex(scratch(i));                                ' Output ROM Data
   Next
   Crc8_check                                               ' Check CRC-8
   Print
   If Scratch(1) <> Ds2438 Then
      Print "No DS2438 connected"
   End If
#endif
   W = 1wirecount()
#if _dbg
   Print "Number of 1wire devices is " ; W
#endif
   Initialize = W
End Function

Sub Measure()
#if _dbg
   Print "Initialize Status Register"
#endif
   _writescratch 0
   1wwrite Measurevdd
#if _dbg
   Print "Read Scratch"
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
      Print Hex(scratch(i)) ; Spc(1);
   Next
   Print
   Print "Copy Statusregister to Memory"
   _copyscratch 0
   _clearscratch 0
   Print "Read Scratch"
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
      Print Hex(scratch(i)) ; Spc(1);
   Next
   Print
   Print "Recall Status Register"
   _recallmemory 0
   Print "Read Scratch"
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
      Print Hex(scratch(i)) ; Spc(1);
   Next
   Print
   Print "Spannungsmessung VDD"
#endif
   _writescratch 0
   1wwrite Measurevdd
   _copyscratch 0
   _convertv
   _recallmemory 0
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
#if _dbg
      Print Hex(scratch(i)) ; Spc(1);
#endif
   Next
   Itemp = Scratch(5)
   Shift Itemp , Left , 8
   Itemp = Itemp + Scratch(4)
   Vdd = Itemp * 0.01
#if _dbg
   Print : Print Vdd ; " V"
   Print "Spannungsmessung VAD"
#endif
   _writescratch 0
   1wwrite Measurevad
   _copyscratch 0
   _convertv
   _recallmemory 0
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
#if _dbg
      Print Hex(scratch(i)) ; Spc(1);
#endif
   Next
   Itemp = Scratch(5)
   Shift Itemp , Left , 8
   Itemp = Itemp + Scratch(4)
   Vad = Itemp * 0.01
#if _dbg
   Print : Print Vad ; " V"
#endif
   Rh_raw = Vad * 161.29
   Rh_raw = Rh_raw / Vdd
   Rh_raw = Rh_raw - 25.8
#if _dbg
   Print "Rel. Luftfeuchte (nicht korrigiert)"
   Print Fusing(rh_raw , "###.#") ; " %"
   Print "Temperaturmessung"
#endif
   _convertt
   _recallmemory 0
   _readscratch 0
   For I = 1 To 9
      Scratch(i) = 1wread()
#if _dbg
      Print Hex(scratch(i)) ; Spc(1);
#endif
   Next
   Itemp = Scratch(3)
   Shift Itemp , Left , 8
   Itemp = Itemp + Scratch(2)
   Itemp = Itemp / 8
   Temp = Itemp * 0.03125
#if _dbg
   Print : Print Fusing(temp , "###.#") ; " grd C"
#endif
   Rh_true = Temp * 0.00216
   Rh_true = 1.0546 - Rh_true
   Rh_true = Rh_raw / Rh_true                               ' RH_true = RH_raw/(1.0546-0.00216*T)
#if _dbg
   Print "Rel. Luftfeuchte (korrigiert)"
   Print Fusing(rh_true , "###.#") ; " %"
#else
   Print Count ; ",";
   Print Fusing(temp , "###.#") ; ",";
   Print Fusing(rh_true , "###.#")
#endif
   Incr Count
   Upperline : Lcd Fusing(temp , "###.#") ; " grd C"
   Lowerline : Lcd Fusing(rh_true , "###.#") ; " %"
End Sub

Sub _convertt()
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Convertt
   Waitms 20
End Sub

Sub _convertv
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Convertv
   Waitms 20
End Sub

Sub _recallmemory(byval Page As Byte)
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Recall_memory                                    ' Recall Page x
   1wwrite Page
End Sub

Sub _readscratch(byval Page As Byte)
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Read_scratch                                     ' Read Scratchpad Page 0
   1wwrite Page
End Sub

Sub _writescratch(byval Page As Byte)
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Write_scratch
   1wwrite Page
End Sub

Sub _copyscratch(byval Page As Byte)
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Copy_scratch                                     ' Copy Scratchpad Page x
   1wwrite Page
End Sub

Sub _clearscratch(byval Page As Byte)
   1wreset                                                  ' Reset 1Wire Device
   If Err = 1 Then Print "1Wire ERROR"
   1wwrite Skip_rom
   1wwrite Write_scratch
   1wwrite Page
   1wwrite &H00
   1wwrite &H11
   1wwrite &H22
   1wwrite &H33
   1wwrite &H44
   1wwrite &H55
   1wwrite &H66
   1wwrite &H77
End Sub

Sub Crc8_check()
   $external _crc8
   Dim Result As Byte
   Loadadr Scratch(1) , Z
   ldi R24, 8
   !Call _crc8
   Loadadr Result , X
   st X, R16
   If Result = 0 Then
      Print " CRC-8 OK";
   Else
      Print " CRC-8 ERROR";
   End If
End Sub